pub struct InfallibleIteratorReadWords<Iter: Iterator> { /* private fields */ }Expand description
Adapter that turns an iterator over Word into a data source.
Wraps an iterator over Word and implements ReadWords<Word, S, ReadError=Infallible> by pulling the iterator each time a client reads from
it. If the iterator implements ExactSizeIterator then this wrapper also implements
BoundedReadWords.
Implements ReadWord for arbitrary Semantics. This is legal since it doesn’t
implement WriteWords, so the question how reads relate to writes is moot.
See also FallibleIteratorReadWords, and module-level documentation for a
detailed usage example.
Implementations
sourceimpl<Iter: Iterator> InfallibleIteratorReadWords<Iter>
impl<Iter: Iterator> InfallibleIteratorReadWords<Iter>
sourcepub fn new<I, Word, ReadError>(iter: I) -> Self
pub fn new<I, Word, ReadError>(iter: I) -> Self
Creates the adapter for the provided iterator.
The provided iterator iter does not need to be fused (i.e., it may return Some
after the first None even though a ReadWords wouldn’t be allowed to do the
equivalent of that). The adapter calls iter.fuse() to ensure correct behavior.
You can get the (fused) iterator back by calling IntoIterator::into_iter.