Trait constriction::backends::IntoSeekReadWords [−][src]
pub trait IntoSeekReadWords<Word, S: Semantics> { type IntoSeekReadWords: Seek + ReadWords<Word, S>; fn into_seek_read_words(self) -> Self::IntoSeekReadWords; }
A trait for types that can be turned into a randomly accessible source of compressed data (for decoders).
This trait is similar to IntoReadWords but it adds the additional guarantee that the
resulting data source implements Seek, i.e., that it can be used by decoders that
support random access.
See also
- module level documentation for more information on the concept of sources and sinks of compressed data;
AsSeekReadWordsfor a simliar conversion that does not take ownership of the data;
Associated Types
type IntoSeekReadWords: Seek + ReadWords<Word, S>[src]
The type of the random-access data source that will result from the conversion.
Required methods
fn into_seek_read_words(self) -> Self::IntoSeekReadWords[src]
Performs the conversion.
Implementors
impl<Word, Buf, S: Semantics> IntoSeekReadWords<Word, S> for Buf where
Buf: AsRef<[Word]> + IntoReadWords<Word, S, IntoReadWords = Cursor<Word, Buf>>,
Cursor<Word, Buf>: ReadWords<Word, S>, [src]
impl<Word, Buf, S: Semantics> IntoSeekReadWords<Word, S> for Buf where
Buf: AsRef<[Word]> + IntoReadWords<Word, S, IntoReadWords = Cursor<Word, Buf>>,
Cursor<Word, Buf>: ReadWords<Word, S>, [src]