pub enum DefaultEncoderFrontendError {
ImpossibleSymbol,
}Variants
ImpossibleSymbol
Tried to encode a symbol with zero probability under the used entropy model.
This error can usually be avoided by using a “leaky” distribution, as the
entropy model, i.e., a distribution that assigns a nonzero probability to all
symbols within a finite domain. Leaky distributions can be constructed with,
e.g., a LeakyQuantizer or with
LeakyCategorical::from_floating_point_probabilities.