Struct constriction::stream::model::LeakilyQuantizedDistribution[][src]

pub struct LeakilyQuantizedDistribution<'q, F, Symbol, Probability, D, const PRECISION: usize> { /* fields omitted */ }
An EntropyModel that approximates a parameterized probability Distribution.

A LeakilyQuantizedDistribution can be created with a LeakyQuantizer. It can be used for encoding and decoding with any of the stream codes provided by the constriction crate (it can only be used for decoding if the underlying Distribution implements the the trait Inverse from the probability crate).

When Should I Use This Type of Entropy Model?

Use a LeakilyQuantizedDistribution when you have a probabilistic model that is defined through some analytic expression (e.g., a mathematical formula for the probability density function of a continuous probability distribution, or a mathematical formula for the probability mass functions of some discrete probability distribution). Examples of probabilistic models that lend themselves to being quantized are continuous distributions such as Gaussian, Laplace, or Exponential, as well as discrete distributions with some analytic expression, such as Binomial.

Do not use a LeakilyQuantizedDistribution if your probabilistic model can only be presented as an explicit probability table. While you could, in principle, apply a LeakyQuantizer to such a Categorical distribution, you will get better computational performance (and also slightly better compression effectiveness) if you instead use one of the dedicated types ContiguousCategoricalEntropyModel, NonContiguousCategoricalEncoderModel, NonContiguousCategoricalDecoderModel, or LookupDecoderModel.

Examples

See examples for LeakyQuantizer.

Computational Efficiency

See discussion for LeakyQuantizer.

Implementations

impl<'q, F, Symbol, Probability, D, const PRECISION: usize> LeakilyQuantizedDistribution<'q, F, Symbol, Probability, D, PRECISION> where
    Probability: BitArray + Into<F>,
    Symbol: PrimInt + AsPrimitive<Probability> + WrappingSub + WrappingAdd,
    F: Float
[src]

pub fn quantizer(&self) -> &'q LeakyQuantizer<F, Symbol, Probability, PRECISION>[src]

Returns the quantizer that was used to create this entropy model.

You may want to reuse this quantizer to quantize further probability distributions.

pub fn inner(&self) -> &D[src]

Returns a reference to the underlying (floating-point) probability Distribution.

Returns the floating-point probability distribution which this LeakilyQuantizedDistribution approximates in fixed-point arithmetic.

See also

pub fn inner_mut(&mut self) -> &mut D[src]

Returns a mutable reference to the underlying (floating-point) probability Distribution.

You can use this method to mutate parameters of the underlying Distribution after it was already quantized. This is safe and cheap since quantization is done lazily anyway. Note that you can’t mutate the support since it is a property of the LeakyQuantizer, not of the Distribution. If you want to modify the support then you have to create a new LeakyQuantizer with a different support.

See also

pub fn into_inner(self) -> D[src]

Consumes the entropy model and returns the underlying (floating-point) probability Distribution.

Returns the floating-point probability distribution which this LeakilyQuantizedDistribution approximates in fixed-point arithmetic.

See also

pub fn support(&self) -> RangeInclusive<Symbol>[src]

Returns the exact range of symbols that have nonzero probability.

See LeakyQuantizer::support.