pub struct RangeDecoder<Word, State, Backend>{ /* private fields */ }Implementations
Sourceimpl<Word, State, Backend> RangeDecoder<Word, State, Backend>
impl<Word, State, Backend> RangeDecoder<Word, State, Backend>
pub fn from_compressed<Buf>(compressed: Buf) -> Result<Self, Backend::ReadError>where
Buf: IntoReadWords<Word, Queue, IntoReadWords = Backend>,
pub fn with_backend(backend: Backend) -> Result<Self, Backend::ReadError>
pub fn for_compressed<'a, Buf>(
compressed: &'a Buf,
) -> Result<Self, Backend::ReadError>where
Buf: AsReadWords<'a, Word, Queue, AsReadWords = Backend>,
Sourcepub fn from_raw_parts(
bulk: Backend,
state: RangeCoderState<Word, State>,
point: State,
) -> Result<Self, Backend>
pub fn from_raw_parts( bulk: Backend, state: RangeCoderState<Word, State>, point: State, ) -> Result<Self, Backend>
Low-level constructor that assembles a RangeDecoder from its internal components.
The arguments bulk, state, and point correspond to the three return values of
the method into_raw_parts.
The construction fails if the argument point lies outside of the range represented
by state. In this case, the method returns the (unmodified) argument bulk back
to the caller, wrapped in an Err variant.
Sourcepub fn into_raw_parts(self) -> (Backend, RangeCoderState<Word, State>, State)
pub fn into_raw_parts(self) -> (Backend, RangeCoderState<Word, State>, State)
Low-level method that disassembles the RangeDecoder into its internal components.
Can be used together with from_raw_parts.
Sourcepub fn maybe_exhausted(&self) -> bool
pub fn maybe_exhausted(&self) -> bool
Same as Decoder::maybe_exhausted, but can be called on a concrete type without
type annotations.