Type Alias constriction::stream::queue::SmallRangeEncoder
source · pub type SmallRangeEncoder<Backend = Vec<u16>> = RangeEncoder<u16, u32, Backend>;Expand description
Type alias for a RangeEncoder for use with lookup models
This encoder has a smaller word size and internal state than DefaultRangeEncoder.
This allows you to use lookup models when decoding data that was encoded with this
coder, see SmallRangeDecoder, as well as SmallContiguousLookupDecoderModel and
SmallNonContiguousLookupDecoderModel.
Aliased Type
struct SmallRangeEncoder<Backend = Vec<u16>> { /* private fields */ }