Struct constriction::stream::stack::AnsCoder[][src]

pub struct AnsCoder<Word, State, Backend = Vec<Word>> where
    Word: BitArray + Into<State>,
    State: BitArray + AsPrimitive<Word>, 
{ /* fields omitted */ }
Entropy coder for both encoding and decoding on a stack.

This is the generic struct for an ANS coder. It provides fine-tuned control over type parameters (see discussion in parent module). You’ll usually want to use this type through the type alias DefaultAnsCoder, which provides sane default settings for the type parameters.

The AnsCoder uses an entropy coding algorithm called range Asymmetric Numeral Systems (rANS). This means that it operates as a stack, i.e., a “last in first out” data structure: encoding “pushes symbols on” the stack and decoding “pops symbols off” the stack in reverse order. In default operation, decoding with an AnsCoder consumes the compressed data for the decoded symbols (however, you can also decode immutable data by using a Cursor). This means that encoding and decoding can be interleaved arbitrarily, thus growing and shrinking the stack of compressed data as you go.

Example

Basic usage example:

use constriction::stream::{model::DefaultLeakyQuantizer, stack::DefaultAnsCoder, Decode};

// `DefaultAnsCoder` is a type alias to `AnsCoder` with sane generic parameters.
let mut ans = DefaultAnsCoder::new();

// Create an entropy model based on a quantized Gaussian distribution. You can use `AnsCoder`
// with any entropy model defined in the `models` module.
let quantizer = DefaultLeakyQuantizer::new(-100..=100);
let entropy_model = quantizer.quantize(probability::distribution::Gaussian::new(0.0, 10.0));

let symbols = vec![-10, 4, 0, 3];
// Encode symbols in *reverse* order, so that we can decode them in forward order.
ans.encode_iid_symbols_reverse(&symbols, &entropy_model).unwrap();

// Obtain temporary shared access to the compressed bit string. If you want ownership of the
// compressed bit string, call `.into_compressed()` instead of `.get_compressed()`.
println!("Encoded into {} bits: {:?}", ans.num_bits(), &*ans.get_compressed().unwrap());

// Decode the symbols and verify correctness.
let reconstructed = ans
    .decode_iid_symbols(4, &entropy_model)
    .collect::<Result<Vec<_>, _>>()
    .unwrap();
assert_eq!(reconstructed, symbols);

Consistency Between Encoding and Decoding

As elaborated in the parent module’s documentation, encoding and decoding operates on a sequence of symbols. Each symbol can be encoded and decoded with its own entropy model (the symbols can even have heterogeneous types). If your goal is to reconstruct the originally encoded symbols during decoding, then you must employ the same sequence of entropy models (in reversed order) during encoding and decoding.

However, using the same entropy models for encoding and decoding is not a general requirement. It is perfectly legal to push (encode) symbols on the AnsCoder using some entropy models, and then pop off (decode) symbols using different entropy models. The popped off symbols will then in general be different from the original symbols, but will be generated in a deterministic way. If there is no deterministic relation between the entropy models used for pushing and popping, and if there is still compressed data left at the end (i.e., if is_empty returns false), then the popped off symbols are, to a very good approximation, distributed as independent samples from the respective entropy models. Such random samples, which consume parts of the compressed data, are useful in the bits-back algorithm.

Implementations

impl<Word, State> AnsCoder<Word, State, Vec<Word>> where
    Word: BitArray + Into<State>,
    State: BitArray + AsPrimitive<Word>, 
[src]

pub fn new() -> Self[src]

Creates an empty ANS entropy coder.

This is usually the starting point if you want to compress data.

Example

let mut ans = constriction::stream::stack::DefaultAnsCoder::new();

// ... push some symbols onto the ANS coder's stack ...

// Finally, get the compressed data.
let compressed = ans.into_compressed();

Generality

To avoid type parameters in common use cases, new is only implemented for AnsCoders with a Vec backend. To create an empty coder with a different backend, call Default::default instead.

impl<Word, State, Backend> AnsCoder<Word, State, Backend> where
    Word: BitArray + Into<State>,
    State: BitArray + AsPrimitive<Word>, 
[src]

pub fn with_state_and_empty_bulk(state: State) -> Self where
    Backend: Default
[src]

pub unsafe fn from_raw_parts(bulk: Backend, state: State) -> Self[src]

Safety

The caller must ensure that state >= State::one() << (State::BITS - Word::BITS) unless bulk.is_empty().

pub fn from_compressed(compressed: Backend) -> Result<Self, Backend> where
    Backend: ReadWords<Word, Stack>, 
[src]

Creates an ANS stack with some initial compressed data.

This is usually the starting point if you want to decompress data previously obtained from into_compressed. However, it can also be used to append more symbols to an existing compressed buffer of data.

Returns Err(compressed) if compressed is not empty and its last entry is zero, since an AnsCoder cannot represent trailing zero words. This error cannot occur if compressed was obtained from into_compressed, which never returns data with a trailing zero word. If you want to construct a AnsCoder from an unknown source of binary data (e.g., to decode some side information into latent variables) then call from_binary instead.

pub fn from_binary(data: Backend) -> Result<Self, Backend::ReadError> where
    Backend: ReadWords<Word, Stack>, 
[src]

Like from_compressed but works on any binary data.

This method is meant for rather advanced use cases. For most common use cases, you probably want to call from_compressed instead.

Different to from_compressed, this method also works if data ends in a zero word. Calling this method is equivalent to (but likely more efficient than) appending a 1 word to data and then calling from_compressed. Note that therefore, this method always constructs a non-empty AnsCoder (even if data is empty):

use constriction::stream::stack::DefaultAnsCoder;

let stack1 = DefaultAnsCoder::from_binary(Vec::new()).unwrap();
assert!(!stack1.is_empty()); // <-- stack1 is *not* empty.

let stack2 = DefaultAnsCoder::from_compressed(Vec::new()).unwrap();
assert!(stack2.is_empty()); // <-- stack2 is empty.

pub fn bulk(&self) -> &Backend[src]

pub fn into_raw_parts(self) -> (Backend, State)[src]

pub fn is_empty(&self) -> bool[src]

Check if no data for decoding is left.

Note that you can still pop symbols off an empty stack, but this is only useful in rare edge cases, see documentation of decode_symbol.

pub fn get_compressed<'a>(
    &'a mut self
) -> Result<impl Deref<Target = Backend> + Debug + Drop + 'a, Backend::WriteError> where
    Backend: ReadWords<Word, Stack> + WriteWords<Word> + Debug
[src]

Assembles the current compressed data into a single slice.

Returns the concatenation of bulk and state. The concatenation truncates any trailing zero words, which is compatible with the constructor from_compressed.

This method requires a &mut self receiver to temporarily append state to bulk (this mutationwill be reversed to recreate the original bulk as soon as the caller drops the returned value). If you don’t have mutable access to the AnsCoder, consider calling iter_compressed instead, or get the bulk and state separately by calling bulk and state, respectively.

The return type dereferences to &[Word], thus providing read-only access to the compressed data. If you need ownership of the compressed data, consider calling into_compressed instead.

Example

use constriction::stream::{
    model::DefaultContiguousCategoricalEntropyModel, stack::DefaultAnsCoder, Decode
};

let mut ans = DefaultAnsCoder::new();

// Push some data on the ans.
let symbols = vec![8, 2, 0, 7];
let probabilities = vec![0.03, 0.07, 0.1, 0.1, 0.2, 0.2, 0.1, 0.15, 0.05];
let model = DefaultContiguousCategoricalEntropyModel
    ::from_floating_point_probabilities(&probabilities).unwrap();
ans.encode_iid_symbols_reverse(&symbols, &model).unwrap();

// Inspect the compressed data.
dbg!(ans.get_compressed());

// We can still use the ANS coder afterwards.
let reconstructed = ans
    .decode_iid_symbols(4, &model)
    .collect::<Result<Vec<_>, _>>()
    .unwrap();
assert_eq!(reconstructed, symbols);

pub fn get_binary<'a>(
    &'a mut self
) -> Result<impl Deref<Target = Backend> + Debug + Drop + 'a, CoderError<(), Backend::WriteError>> where
    Backend: ReadWords<Word, Stack> + WriteWords<Word> + Debug
[src]

pub fn iter_compressed<'a>(&'a self) -> impl Iterator<Item = Word> + '_ where
    &'a Backend: IntoIterator<Item = &'a Word>, 
[src]

Iterates over the compressed data currently on the ans.

In contrast to get_compressed or into_compressed, this method does not require mutable access or even ownership of the AnsCoder.

Example

use constriction::stream::{model::DefaultLeakyQuantizer, stack::DefaultAnsCoder, Decode};

// Create a stack and encode some stuff.
let mut ans = DefaultAnsCoder::new();
let symbols = vec![8, -12, 0, 7];
let quantizer = DefaultLeakyQuantizer::new(-100..=100);
let model =
    quantizer.quantize(probability::distribution::Gaussian::new(0.0, 10.0));
ans.encode_iid_symbols_reverse(&symbols, &model).unwrap();

// Iterate over compressed data, collect it into to a Vec``, and compare to direct method.
let compressed_iter = ans.iter_compressed();
let compressed_collected = compressed_iter.collect::<Vec<_>>();
assert!(!compressed_collected.is_empty());
assert_eq!(compressed_collected, *ans.get_compressed().unwrap());

pub fn num_words(&self) -> usize where
    Backend: BoundedReadWords<Word, Stack>, 
[src]

Returns the number of compressed words on the ANS coder’s stack.

This includes a constant overhead of between one and two words unless the stack is completely empty.

This method returns the length of the slice, the Vec<Word>, or the iterator that would be returned by get_compressed, into_compressed, or iter_compressed, respectively, when called at this time.

See also num_bits.

pub fn num_bits(&self) -> usize where
    Backend: BoundedReadWords<Word, Stack>, 
[src]

pub fn num_valid_bits(&self) -> usize where
    Backend: BoundedReadWords<Word, Stack>, 
[src]

pub fn into_decoder(self) -> AnsCoder<Word, State, Backend::IntoReadWords> where
    Backend: IntoReadWords<Word, Stack>, 
[src]

pub fn into_seekable_decoder(
    self
) -> AnsCoder<Word, State, Backend::IntoSeekReadWords> where
    Backend: IntoSeekReadWords<Word, Stack>, 
[src]

Consumes the AnsCoder and returns a decoder that implements Seek.

This method is similar to seekable_decoder except that it takes ownership of the original AnsCoder, so the returned seekable decoder can typically be returned from the calling function or put on the heap.

pub fn as_decoder<'a>(&'a self) -> AnsCoder<Word, State, Backend::AsReadWords> where
    Backend: AsReadWords<'a, Word, Stack>, 
[src]

pub fn as_seekable_decoder<'a>(
    &'a self
) -> AnsCoder<Word, State, Backend::AsSeekReadWords> where
    Backend: AsSeekReadWords<'a, Word, Stack>, 
[src]

Returns a decoder that implements Seek.

The returned decoder shares access to the compressed data with the original AnsCoder (i.e., self). This means that:

  • you can call this method several times to create several seekable decoders with independent views into the same compressed data;
  • once the lifetime of all handed out seekable decoders ends, the original AnsCoder can be used again; and
  • the constructed seekable decoder cannot outlive the original AnsCoder; for example, if the original AnsCoder lives on the calling function’s call stack frame then you cannot return the constructed seekable decoder from the calling function. If this is a problem then call into_seekable_decoder instead.

Limitations

TODO: this text is outdated.

This method is only implemented for AnsCoders whose backing store of compressed data (Backend) implements AsRef<[Word]>. This includes the default backing data store Backend = Vec<Word>.

impl<Word, State> AnsCoder<Word, State> where
    Word: BitArray + Into<State>,
    State: BitArray + AsPrimitive<Word>, 
[src]

pub fn clear(&mut self)[src]

Discards all compressed data and resets the coder to the same state as Coder::new.

impl<'bulk, Word, State> AnsCoder<Word, State, Cursor<Word, &'bulk [Word]>> where
    Word: BitArray + Into<State>,
    State: BitArray + AsPrimitive<Word>, 
[src]

pub fn from_compressed_slice(compressed: &'bulk [Word]) -> Result<Self, ()>[src]

pub fn from_binary_slice(data: &'bulk [Word]) -> Self[src]

impl<Word, State, Buf> AnsCoder<Word, State, Reverse<Cursor<Word, Buf>>> where
    Word: BitArray + Into<State>,
    State: BitArray + AsPrimitive<Word>,
    Buf: AsRef<[Word]>, 
[src]

pub fn from_reversed_compressed(compressed: Buf) -> Result<Self, Buf>[src]

pub fn from_reversed_binary(data: Buf) -> Self[src]

impl<Word, State, Iter, ReadError> AnsCoder<Word, State, FallibleIteratorReadWords<Iter>> where
    Word: BitArray + Into<State>,
    State: BitArray + AsPrimitive<Word>,
    Iter: Iterator<Item = Result<Word, ReadError>>,
    FallibleIteratorReadWords<Iter>: ReadWords<Word, Stack, ReadError = ReadError>, 
[src]

pub fn from_reversed_compressed_iter(
    compressed: Iter
) -> Result<Self, Fuse<Iter>>
[src]

pub fn from_reversed_binary_iter(data: Iter) -> Result<Self, ReadError>[src]

impl<Word, State, Backend> AnsCoder<Word, State, Backend> where
    Word: BitArray + Into<State>,
    State: BitArray + AsPrimitive<Word>,
    Backend: WriteWords<Word>, 
[src]

pub fn encode_symbols_reverse<S, M, I, const PRECISION: usize>(
    &mut self,
    symbols_and_models: I
) -> Result<(), CoderError<DefaultEncoderFrontendError, Backend::WriteError>> where
    S: Borrow<M::Symbol>,
    M: EncoderModel<PRECISION>,
    M::Probability: Into<Word>,
    Word: AsPrimitive<M::Probability>,
    I: IntoIterator<Item = (S, M)>,
    I::IntoIter: DoubleEndedIterator
[src]

pub fn try_encode_symbols_reverse<S, M, E, I, const PRECISION: usize>(
    &mut self,
    symbols_and_models: I
) -> Result<(), TryCodingError<CoderError<DefaultEncoderFrontendError, Backend::WriteError>, E>> where
    S: Borrow<M::Symbol>,
    M: EncoderModel<PRECISION>,
    M::Probability: Into<Word>,
    Word: AsPrimitive<M::Probability>,
    I: IntoIterator<Item = Result<(S, M), E>>,
    I::IntoIter: DoubleEndedIterator
[src]

pub fn encode_iid_symbols_reverse<S, M, I, const PRECISION: usize>(
    &mut self,
    symbols: I,
    model: M
) -> Result<(), CoderError<DefaultEncoderFrontendError, Backend::WriteError>> where
    S: Borrow<M::Symbol>,
    M: EncoderModel<PRECISION> + Copy,
    M::Probability: Into<Word>,
    Word: AsPrimitive<M::Probability>,
    I: IntoIterator<Item = S>,
    I::IntoIter: DoubleEndedIterator
[src]

pub fn into_compressed(self) -> Result<Backend, Backend::WriteError>[src]

Consumes the ANS coder and returns the compressed data.

The returned data can be used to recreate an ANS coder with the same state (e.g., for decoding) by passing it to from_compressed.

If you don’t want to consume the ANS coder, consider calling get_compressed, iter_compressed instead.

Example

use constriction::stream::{
    model::DefaultContiguousCategoricalEntropyModel, stack::DefaultAnsCoder, Decode
};

let mut ans = DefaultAnsCoder::new();

// Push some data onto the ANS coder's stack:
let symbols = vec![8, 2, 0, 7];
let probabilities = vec![0.03, 0.07, 0.1, 0.1, 0.2, 0.2, 0.1, 0.15, 0.05];
let model = DefaultContiguousCategoricalEntropyModel
    ::from_floating_point_probabilities(&probabilities).unwrap();
ans.encode_iid_symbols_reverse(&symbols, &model).unwrap();

// Get the compressed data, consuming the ANS coder:
let compressed = ans.into_compressed().unwrap();

// ... write `compressed` to a file and then read it back later ...

// Create a new ANS coder with the same state and use it for decompression:
let mut ans = DefaultAnsCoder::from_compressed(compressed).expect("Corrupted compressed file.");
let reconstructed = ans
    .decode_iid_symbols(4, &model)
    .collect::<Result<Vec<_>, _>>()
    .unwrap();
assert_eq!(reconstructed, symbols);
assert!(ans.is_empty())

pub fn into_binary(self) -> Result<Backend, Option<Backend::WriteError>>[src]

Returns the binary data if it fits precisely into an integer number of Words

This method is meant for rather advanced use cases. For most common use cases, you probably want to call into_compressed instead.

This method is the inverse of from_binary. It is equivalent to calling into_compressed, verifying that the returned vector ends in a 1 word, and popping off that trailing 1 word.

Returns Err(()) if the compressed data (excluding an obligatory trailing 1 bit) does not fit into an integer number of Words. This error case includes the case of an empty AnsCoder (since an empty AnsCoder lacks the obligatory trailing one-bit).

Example

// Some binary data we want to represent on a `AnsCoder`.
let data = vec![0x89ab_cdef, 0x0123_4567];

// Constructing a `AnsCoder` with `from_binary` indicates that all bits of `data` are
// considered part of the information-carrying payload.
let stack1 = constriction::stream::stack::DefaultAnsCoder::from_binary(data.clone()).unwrap();
assert_eq!(stack1.clone().into_binary().unwrap(), data); // <-- Retrieves the original `data`.

// By contrast, if we construct a `AnsCoder` with `from_compressed`, we indicate that
// - any leading `0` bits of the last entry of `data` are not considered part of
//   the information-carrying payload; and
// - the (obligatory) first `1` bit of the last entry of `data` defines the
//   boundary between unused bits and information-carrying bits; it is therefore
//   also not considered part of the payload.
// Therefore, `stack2` below only contains `32 * 2 - 7 - 1 = 56` bits of payload,
// which cannot be exported into an integer number of `u32` words:
let stack2 = constriction::stream::stack::DefaultAnsCoder::from_compressed(data.clone()).unwrap();
assert!(stack2.clone().into_binary().is_err()); // <-- Returns an error.

// Use `into_compressed` to retrieve the data in this case:
assert_eq!(stack2.into_compressed().unwrap(), data);

// Calling `into_compressed` on `stack1` would append an extra `1` bit to indicate
// the boundary between information-carrying bits and padding `0` bits:
assert_eq!(stack1.into_compressed().unwrap(), vec![0x89ab_cdef, 0x0123_4567, 0x0000_0001]);

impl<Word, State, Buf> AnsCoder<Word, State, Cursor<Word, Buf>> where
    Word: BitArray,
    State: BitArray + AsPrimitive<Word> + From<Word>,
    Buf: AsRef<[Word]> + AsMut<[Word]>, 
[src]

pub fn into_reversed(self) -> AnsCoder<Word, State, Reverse<Cursor<Word, Buf>>>[src]

impl<Word, State, Buf> AnsCoder<Word, State, Reverse<Cursor<Word, Buf>>> where
    Word: BitArray,
    State: BitArray + AsPrimitive<Word> + From<Word>,
    Buf: AsRef<[Word]> + AsMut<[Word]>, 
[src]

pub fn into_reversed(self) -> AnsCoder<Word, State, Cursor<Word, Buf>>[src]