Struct constriction::symbol::SymbolCoder[][src]

pub struct SymbolCoder<Word: BitArray, S: Semantics, B = Vec<Word>> { /* fields omitted */ }
Generic symbol coder for 3 out of 4 possible cases

You likely won’t spell out this type explicitly. It’s more convenient to use one of the type aliases QueueEncoder or StackCoder (or the more opinionated aliases DefaultQueueEncoder and DefaultStackCoder).

Depending on the type parameter S, this type supports:

  • encoding on a queue (i.e., writing prefix codes); this is type aliased as QueueEncoder.
  • encoding and decoding on a stack (i.e., writing suffix codes and reading them back in reoverse order). This is type aliased as StackCoder.

This type does not support decoding from a queue. Use a QueueDecoder for this purpose.

Implementations

Returns the correct len for, e.g., B = Vec<Word> regardless of whether S = Queue or S = Stack.

Returns true if no bits are on the SymbolCoder.

Errors
  • Returns Err(CoderError::FrontendError(compressed)) if compressed ends in a zero which is not allowed for stacks (because we need a terminal “one” bit to identify the head position). Note that “EOF” is not considered an error, and the method will return with a success if called with an empty backend.
  • Returns Err(CoderError::BackendError(err)) if reading the last word from the backend resulted in Err(err).
  • Returns Ok(stack_coder) in all other cases, including the case where the backend is empty.

Consumes the coder and returns an iterator over bits (in reverse direction)

You often don’t need to call this method since a StackCoder is already an iterator if the backend implements ReadWords<Word, Stack> (as the default backend Vec<Word> does).

Returns an iterator over bits (in reverse direction) that leaves the current coder untouched.

You often don’t need to call this method since a StackCoder is already an iterator if the backend implements ReadWords<Word, Stack> (as the default backend Vec<Word> does).