Struct constriction::Queue
source · pub struct Queue {}Expand description
Zero sized marker trait for first-in-first-out read/write Semantics
This type typically only comes up in advanced use cases that are generic over read/write
semantics. If you are looking for an entropy coder that operates as a queue, check out
the module stream::queue.